@@ -1,7 +1,7 @@
 /*
     ccid.h: CCID structures
     Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2015   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2017   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -175,6 +175,7 @@ typedef struct
 
 /* Features from dwFeatures */
 #define CCID_CLASS_AUTO_CONF_ATR	0x00000002
+#define CCID_CLASS_AUTO_ACTIVATION	0x00000004
 #define CCID_CLASS_AUTO_VOLTAGE		0x00000008
 #define CCID_CLASS_AUTO_BAUD		0x00000020
 #define CCID_CLASS_AUTO_PPS_PROP	0x00000040
@@ -221,6 +222,7 @@ typedef struct
 #define SCR331DI	0x04E65111
 #define SCR331DINTTCOM	0x04E65120
 #define SDI010		0x04E65121
+#define SEC1210	0x04241202
 #define CHERRYXX33	0x046A0005
 #define CHERRYST2000	0x046A003E
 #define OZ776		0x0B977762
@@ -244,9 +246,15 @@ typedef struct
 #define FEITIANR502DUAL 0x096E060D
 #define MICROCHIP_SEC1100 0x04241104
 #define CHERRY_KC1000SC 0x046A00A1
-#define ElatecTWN4	0x09D80427
+#define ElatecTWN4_CCID_CDC	0x09D80427
+#define ElatecTWN4_CCID	0x09D80428
 #define SCM_SCL011 0x04E65293
 #define HID_AVIATOR	0x076B3A21
+#define HID_OMNIKEY_5422 0x076B5422
+#define HID_OMNIKEY_3X21 0x076B3031 /* OMNIKEY 3121 or 3021 or 1021 */
+#define HID_OMNIKEY_6121 0x076B6632 /* OMNIKEY 6121 */
+#define CHERRY_XX44	0x046A00A7 /* Cherry Smart Terminal xx44 */
+#define FUJITSU_D323 0x0BF81024 /* Fujitsu Smartcard Reader D323 */
 
 // CCID readers
 #define ACS_ACR32_ICC_READER			0x072fb301
@@ -273,6 +281,7 @@ typedef struct
 #define ACS_ACR1251K_DUAL_READER		0x072f2232
 #define ACS_ACR1251_1S_DUAL_READER		0x072f2242
 #define ACS_ACR1252_1S_CL_READER		0x072f223b
+#define ACS_ACR1252IMP_1S_CL_READER		0x072f2259
 #define ACS_ACR1261_1S_DUAL_READER		0x072f2211
 #define ACS_ACR128U						0x072f2100
 #define ACS_ACR1281_DUAL_READER_QPBOC	0x072f2208
@@ -290,6 +299,7 @@ typedef struct
 #define ACS_AET62_PICC_READER			0x072f0102
 #define ACS_AET62_1SAM_PICC_READER		0x072f0103
 #define ACS_APG8201						0x072f8201
+#define ACS_APG8201_B2					0x072f8206
 #define ACS_APG8201Z					0x072f8202
 #define ACS_APG8201Z2					0x072f8205
 
@@ -317,12 +327,12 @@ typedef struct
 /*
  * Possible values :
  * 3 -> 1.8V, 3V, 5V
- * 2 -> 3V, 5V
- * 1 -> 5V only
+ * 2 -> 3V, 5V, 1.8V
+ * 1 -> 5V, 1.8V, 3V
  * 0 -> automatic (selection made by the reader)
  */
 /*
- * To be safe we default to 5V
+ * The default is to start at 5V
  * otherwise we would have to parse the ATR and get the value of TAi (i>2) when
  * in T=15
  */
