@@ -1,7 +1,7 @@
 /*
 	ccid_usb.c: USB access routines using the libusb library
 	Copyright (C) 2003-2010	Ludovic Rousseau
-	Copyright (C) 2009-2016	Advanced Card Systems Ltd.
+	Copyright (C) 2009-2017	Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -296,6 +296,21 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 				DEBUG_COMM3("usb bus/device: %d/%d", device_bus, device_addr);
 			}
 		}
+		else
+		{
+			/* format usb:%04x/%04x:libusb-1.0:%d:%d:%d */
+			if ((dirname = strstr(device, "libusb-1.0:")) != NULL)
+			{
+				/* convert the interface number, bus and device ids */
+				if (sscanf(dirname + 11, "%d:%d:%d",
+					&device_bus, &device_addr, &interface_number) == 3)
+				{
+					DEBUG_COMM2("interface_number: %d", interface_number);
+					DEBUG_COMM3("usb bus/device: %d/%d", device_bus,
+						device_addr);
+				}
+			}
+		}
 	}
 #endif
 
@@ -437,12 +452,23 @@ again_libusb:
 				int numSlots = 0;
 
 #ifdef USE_COMPOSITE_AS_MULTISLOT
-				static int static_interface = 1;
+				/* use the first CCID interface on first call */
+				static int static_interface = -1;
+				int max_interface_number = 2;
 
 				/* simulate a composite device as when libudev is used */
 				if ((GEMALTOPROXDU == readerID)
 					|| (GEMALTOPROXSU == readerID)
-					|| (FEITIANR502DUAL == readerID))
+					|| (HID_OMNIKEY_5422 == readerID)
+					|| (FEITIANR502DUAL == readerID)
+					|| (ACS_ACR1281_DUAL_READER_QPBOC == readerID)
+					|| (ACS_ACR1281_DUAL_READER_BSI == readerID)
+					|| (ACS_ACR1281_1S_PICC_READER == readerID)
+					|| (ACS_ACR1251_1S_CL_READER == readerID)
+					|| (ACS_ACR1251U_C == readerID)
+					|| (ACS_ACR1251K_DUAL_READER == readerID)
+					|| (ACS_ACR1252_1S_CL_READER == readerID)
+					|| (ACS_ACR1252IMP_1S_CL_READER == readerID))
 				{
 						/*
 						 * We can't talk to the two CCID interfaces
@@ -464,20 +490,33 @@ again_libusb:
 	* 1: Gemalto Prox-DU [Prox-DU Contactless_09A00795] (09A00795) 01 00
 						 */
 
-					/* the CCID interfaces are 1 and 2 */
+					/* for the Gemalto Prox-DU/SU the interfaces are:
+					 * 0: Prox-DU HID (not used)
+					 * 1: Prox-DU Contactless (CCID)
+					 * 2: Prox-DU Contact (CCID)
+					 *
+					 * For the Feitian R502 the interfaces are:
+					 * 0: R502 Contactless Reader (CCID)
+					 * 1: R502 Contact Reader (CCID)
+					 * 2: R502 SAM1 Reader (CCID)
+					 *
+					 * For the HID Omnikey 5422 the interfaces are:
+					 * 0: OMNIKEY 5422CL Smartcard Reader
+					 * 1: OMNIKEY 5422 Smartcard Reader
+					 */
 					interface_number = static_interface;
-				}
-				// Simulate ACR1281 Dual Reader (composite device) as multi-slot reader
-				else if ((ACS_ACR1281_DUAL_READER_QPBOC == readerID) ||
-					(ACS_ACR1281_DUAL_READER_BSI == readerID) ||
-					(ACS_ACR1281_1S_PICC_READER == readerID) ||
-					(ACS_ACR1251_1S_CL_READER == readerID) ||
-					(ACS_ACR1251U_C == readerID) ||
-					(ACS_ACR1251K_DUAL_READER == readerID) ||
-					(ACS_ACR1252_1S_CL_READER == readerID))
-				{
-					// the CCID interfaces are 0 and 1
-					interface_number = static_interface - 1;
+
+					if ((HID_OMNIKEY_5422 == readerID)
+						|| (ACS_ACR1281_DUAL_READER_QPBOC == readerID)
+						|| (ACS_ACR1281_DUAL_READER_BSI == readerID)
+						|| (ACS_ACR1281_1S_PICC_READER == readerID)
+						|| (ACS_ACR1251_1S_CL_READER == readerID)
+						|| (ACS_ACR1251U_C == readerID)
+						|| (ACS_ACR1251K_DUAL_READER == readerID)
+						|| (ACS_ACR1252_1S_CL_READER == readerID)
+						|| (ACS_ACR1252IMP_1S_CL_READER == readerID))
+						/* only 2 interfaces for this device */
+						max_interface_number = 1;
 				}
 #endif
 				/* is it already opened? */
@@ -727,6 +766,7 @@ again:
 #ifdef USE_COMPOSITE_AS_MULTISLOT
 				if ((GEMALTOPROXDU == readerID)
 					|| (GEMALTOPROXSU == readerID)
+					|| (HID_OMNIKEY_5422 == readerID)
 					|| (FEITIANR502DUAL == readerID)
 					|| (ACS_ACR1281_DUAL_READER_QPBOC == readerID)
 					|| (ACS_ACR1281_DUAL_READER_BSI == readerID)
@@ -734,14 +774,16 @@ again:
 					|| (ACS_ACR1251_1S_CL_READER == readerID)
 					|| (ACS_ACR1251U_C == readerID)
 					|| (ACS_ACR1251K_DUAL_READER == readerID)
-					|| (ACS_ACR1252_1S_CL_READER == readerID))
+					|| (ACS_ACR1252_1S_CL_READER == readerID)
+					|| (ACS_ACR1252IMP_1S_CL_READER == readerID))
 				{
 					/* use the next interface for the next "slot" */
-					static_interface++;
+					static_interface = interface + 1;
 
 					/* reset for a next reader */
-					if (static_interface > 2)
-						static_interface = (FEITIANR502DUAL == readerID) ? 0: 1;
+					/* max interface number for all 3 readers is 2 */
+					if (static_interface > max_interface_number)
+						static_interface = -1;
 				}
 #endif
 
@@ -982,7 +1024,8 @@ again:
 				if ((readerID == ACS_ACR1281_1S_PICC_READER) ||
 					(readerID == ACS_ACR1251_1S_CL_READER) ||
 					(readerID == ACS_ACR1251U_C) ||
-					(readerID == ACS_ACR1252_1S_CL_READER))
+					(readerID == ACS_ACR1252_1S_CL_READER) ||
+					(readerID == ACS_ACR1252IMP_1S_CL_READER))
 				{
 					if (interface == 1)
 						usbDevice[reader_index].ccid.isSamSlot = TRUE;
@@ -1719,7 +1762,7 @@ int InterruptRead(int reader_index, int timeout /* in ms */)
 		libusb_free_transfer(transfer);
 		DEBUG_CRITICAL2("libusb_submit_transfer failed: %s",
 			libusb_error_name(ret));
-		return ret;
+		return IFD_COMMUNICATION_ERROR;
 	}
 
 	usbDevice[reader_index].polling_transfer = transfer;
@@ -1738,7 +1781,7 @@ int InterruptRead(int reader_index, int timeout /* in ms */)
 			libusb_free_transfer(transfer);
 			DEBUG_CRITICAL2("libusb_handle_events failed: %s",
 				libusb_error_name(ret));
-			return ret;
+			return IFD_COMMUNICATION_ERROR;
 		}
 	}
 
